#Copyright Massachusetts Institute of technology, 1998.
#Written by Eric Prud'hommeaux for the World Wide Web Consortium

use strict;
require Exporter;
require AutoLoader;

$W3C::Util::Properties::REVISION = '$Id: Properties.pm,v 1.10 1999/12/14 13:14:41 eric Exp $ ';

package W3C::Util::Properties;
use W3C::Util::Exception;
use vars qw($VERSION $DSLI @ISA @EXPORT @EXPORT_OK);
@ISA = qw(Exporter AutoLoader);
@EXPORT = qw();
@EXPORT_OK = qw();
$VERSION = 0.11;
$DSLI = 'adpO';

sub new {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my $self  = {};

    bless ($self, $class);
    $self->clear;
    $self->slurp(@_);
    return $self;
}

sub clear {
    my ($self, $names) = @_;
    if (defined $names) {
	$names = [$names] if (ref $names ne 'ARRAY');
	foreach my $name (@$names) {
	    $self->{NOCASE}{$name} = {};
	    $self->{PROPS}{$name} = {};
	}
    } else {
	$self->{NOCASE} = {};
	$self->{PROPS} = {};
    }
}

sub read {
    my $self = shift;
    my $lines = shift;
    foreach my $line (@$lines) {
	$line =~ s/\s*\#.*\Z//; # cheap (unescapable) commenting scheme
	chomp $line;
	next if (!$line);
	my ($name, $value) = split(':\s*', $line, 2);
	$self->put($name, $value);
    }
}

sub put {
    my $self = shift;
    my $name = shift;
    my $value = shift;
    push (@{$self->{PROPS}{$name}}, $value);
    $self->{NOCASE}{lc $name} = $name;
}

sub putR {
    my $self = shift;
    my $name = shift;
    my $value = shift;
    $self->{PROPS}{$name} = [$value];
    $self->{NOCASE}{lc $name} = $name;
}

#####
# slurp - read attribute - value pairs from an array of initializers
#
# args - each initializer may be a ref to a
#  HASH - read attribute names and values from hash
#  ARRAY - prompt names and read values from STDIN
#  SCALAR - read name:value\n pairs from the scalar
#  or a strait (unref'd) filename - read name:value\n pairs from file

sub slurp {
    my $self = shift;
    foreach my $initializer (@_) {
	if (ref $initializer eq 'HASH') {
	    foreach my $name (keys %$initializer) {
		$self->put($name, $initializer->{$name});
	    }
	} elsif (ref $initializer eq 'ARRAY') {
	    foreach my $name (@$initializer) {
		$self->put($name, $self->readStdin($name, $name eq 'password'));
	    }
	} elsif (ref $initializer eq 'SCALAR') {
	    my @lines = split("\n", $$initializer);
	    $self->read(\@lines);
	} else {
	    my $propertyFileName = $initializer;
	    local *PROPS;
	    open (PROPS, $propertyFileName) || &throw(new W3C::Util::FileNotFoundException({-filename => $propertyFileName}));
	    my @lines = (<PROPS>);
	    close (PROPS);
	    $self->read(\@lines);
	}
    }
}

sub defaultTo {
    my $self = shift;
    my $name = shift;
    my $value = shift;
    $self->put($name, $value) if (!defined $self->get($name));
}

sub defaultToI {
    my $self = shift;
    my $name = shift;
    my $value = shift;
    $self->put($name, $value) if (!defined $self->getI($name));
}

sub get {
    my $self = shift;
    my $name = shift;
    return $self->{PROPS} if (!defined $name);
    return wantarray ? (@{$self->{PROPS}{$name}}) : $self->{PROPS}{$name}[0];
}

sub getI {
    my $self = shift;
    my $caseSensitiveName = $self->{NOCASE}{lc shift};
    return defined $caseSensitiveName ? wantarray ? ($self->get($caseSensitiveName)) : $self->get($caseSensitiveName) : undef;
}

sub readStdin {
    my $self = shift;
    my $name = shift;
    my $hidden = shift;
    print $name,': ';
    system('stty -echo') if ($hidden);
    my $value = <STDIN>;
    (system('stty echo') && print "\n") if ($hidden);
    return $value;
}

sub show {
    my ($self, $prefix) = @_; $prefix = '  ' if (!defined $prefix);
    my $ret = $prefix.$self."\n";
    foreach my $key (keys %{$self->{PROPS}}) {
	$ret .= $prefix.'  '.$key.': [';
	my $array = $self->{PROPS}{$key};
	foreach my $element (@$array) {
	    $ret .= '"'.$element.'", ';
	}
	$ret .= "]\n";
    }
    return $ret;
}

1;

__END__

=head1 NAME

W3C::Util::Properties - tool to store and recover name/attribute pairs

=head1 SYNOPSIS

  use W3C::Util::Properties;
  my $prop = new W3C::Util::Properties($sourceFile);

=head1 DESCRIPTION

Tool to store and recover name/attribute pairs.

This module is part of the W3C::Utils CPAN module.

=head1 AUTHOR

Eric Prud\'hommeaux <eric@w3.org>

=head1 SEE ALSO

perl(1).

=cut
